var Articles = new Class({
  Implements: [Options],

  options: {
    'artilesSelector': "#articles-pagenator .p-article",
    'prevSelector': "#articles-pagenator .pagenator .nextleft",
    'nextSelector': "#articles-pagenator .pagenator .nextright"
  },

  initialize: function(options){
    this.setOptions(options);
    this._cur = 0;
    this._articles = $$(this.options.artilesSelector);
    this._length = this._articles.length;
    this.next = $$(this.options.nextSelector);
    this.prev = $$(this.options.prevSelector);
    this.popup = new ArticlePopup({'popup_id': 'popup-article',
                                   'overlaySelector': '.overLayer-article' });

    this.attachEvents();
    if (window.location.hash){
      var re = /article-(\d+)/i;
      var match = window.location.hash.match(re);
      if(match && match.length==2){
        var articleIndex = this.getIndexById(match[1]);
        
        if (articleIndex != null){
          this.scrollTo(articleIndex);
          this.show(articleIndex);
        }
      };
    } else
      this.scrollTo(this._cur);
  },

  getIndexById: function(id){
    var article = this._articles.filter("[rel=article-"+id+"]");
    if (article && article.length>0){
      return this._articles.indexOf(article[0]);
    }
    else{
      return null;
    }
  },

  scrollTo: function(index){
    index = Number(index);
    this._articles.addClass('hide');
    this._articles[index].removeClass('hide');
    this._cur = index;
  },

  scrollToNext: function(){
    if (this._cur+1>=this._length){
      this.scrollTo(0)
    } else {
      this.scrollTo(this._cur+1);
    }
  },

  scrollToPrev: function(){
    if (this._cur<=0){
      this.scrollTo(this._length-1)
    } else {
      this.scrollTo(this._cur-1);
    }
  },

  attachEvents: function(){
    var articlesObj = this;
    this.next.addEvent('click', function(e){
      e.stop();
      articlesObj.scrollToNext();
    });
    this.prev.addEvent('click', function(e){
      e.stop();
      articlesObj.scrollToPrev();
    });
    this._articles.addEvent("click", function(e){
      e.stop();
      articlesObj._showPopup(this)
    });

    this.popup.addEvent('close', function(){
      window.location.hash = "index";
    })
  },

  show: function(index){
    var target = this._articles[index];
    this._showPopup(target);
  },

  _showPopup: function(target){
    var popup = this.popup;
    popup.clear();
    popup.set_title(target.get('data-title'));
    if (target.get('data-src')) {
      popup.set_img(new Element('img', {
          'src': target.get('data-src')
      }));
      if (target.get('data-img-title')) {
        popup.set_img_description(new Element('p', {
          'html': target.get('data-img-title')
        }))
      }
    }
    popup.set_body(new Element('p', {
        'text': target.get('data-annonce')
    }));
    if (target.get('data-annonce')) {
      popup.set_annonce(target.get('data-annonce'));
    }
    if (target.get('data-date')) {
      popup.set_date(target.get('data-date'));
    }
    if (target.get('data-body')) {
      if(target.get('data-body-text')){
          popup.set_body(new Element('div',{
                        'class': 'body-text',
                        'html': target.get('data-body')}));
      }else{
          popup.set_body(target.get('data-body'));
      }
    }
    popup.set_anchor(target.get('rel'));
    popup.show();
    
    var re = /article-(\d+)/i;
    window.location.hash = "article-"+(target.get("rel").match(re)[1]);
  }
});



