var SimpleCalendar = new Class({
  Implements: Options,
  Binds: ["update", 'fillSelects'],

  options: {
    yearSelector: '.calendar .dates select[name="year"]',
    monthSelector: '.calendar .dates select[name="month"]',
    calendarSelector: '.calendar',
    showByDefault: true,
    monthNames: ["Январь", "Февраль", "Март",  "Апрель", "Мая", "Июнь", "Июль", "Август", "Сентябрь", "Октябрь", "Ноябрь", "Декабрь"],
    avalibleMonths: new Hash({2011: [11, 12], 2012:[1, 2]})
  },


  update: function(){
    var year = this.yearSelect.get('value');
    var month = this.monthSelect.get('value');
    this.calLists.addClass('hide');
    $('cal-'+year+'-'+month).removeClass('hide');
  },

  hide: function(){
    this.dates.addClass('hide');
    this.opened = false;
  },

  show: function(){
    this.dates.removeClass('hide');
    this.opened=true;
    this.update();
  },

  toggle: function(){
    if (this.opened)
      this.hide();
    else
      this.show();
  },

  _setYears: function(defaultYear){
    var calendar = this;
    calendar.yearSelect.getChildren().destroy();
    this.options.avalibleMonths.getKeys().sort().each(function(year){
      calendar.yearSelect.grab(new Element('option', {'value':year, 'html':year}))
    });
    if (defaultYear)
      this.yearSelect.set('value', defaultYear);
  },

  _setMonths: function(defaultMonth) {
    var calendar = this;
    var year = this.yearSelect.get('value');

    // Here we update months select
    calendar.monthSelect.getChildren().destroy();
    this.options.avalibleMonths[year].sort().each(function(month){
      calendar.monthSelect.grab(new Element('option', {'value':month, 'html':calendar.options.monthNames[month-1]}))
    });
    if (defaultMonth)
      this.monthSelect.set('value', defaultMonth);
  },

  _attachEvents: function(){
    var calendar = this;
    $$("a.toggle").addEvent("click", function(e){
      e.stop;
      calendar.toggle();
    })
    this.yearSelect.addEvent('change', function(e){
      e.stop()
      calendar._setMonths()
      calendar.update();
    });
    this.monthSelect.addEvent('change', this.update);
  },

  initialize: function(options, curDate){
    
    this.setOptions(options);

    this.calendar = $$(this.options.calendarSelector);
    this.dates = this.calendar.getElements('.dates')[0];
    this.opened=true;

    this.yearSelect = $$(this.options.yearSelector)[0];
    this.monthSelect = $$(this.options.monthSelector)[0];
    this.calLists = $$('.cal-body');
    this.calLists.addClass('hide');
    
    if (! curDate){
      // XXX: Get last avalible date from options.avalibleMonth
      var lastYear = this.options.avalibleMonths.getKeys().sort().getLast();
      var lastMonth = this.options.avalibleMonths[lastYear].sort().getLast();
      curDate = [lastYear, lastMonth];
    }
    this._setYears(curDate[0]);
    this._setMonths(curDate[1]);

    // XXX: Update caled twice on start. Fix it;
    if (this.options.showByDefault)
      this.show();
    else
      this.hide();
   
    this._attachEvents();
  }
});
