var Popup = new Class({
  Implements: [Options, Events],
  
  options: {
    'popup_id': 'popup',
    'dragable': false,
    'close_by_overlay': true,
    'adopt_size': true,
    'style_class': '',
    'overlaySelector': ".overLayer",
    'min_width': 780,
    'min_height': 100
  },

  initialize: function(options){
    this.setOptions(options);
    this.popup = $(this.options.popup_id);
    this.popup_title = this.popup.getElement('.popup_title');
    this.popup_body = this.popup.getElement('.popup_body');
    this.popup_img = this.popup.getElement('.img .image');
    this.popup_img_description = this.popup.getElement('.img .description');
    this.overlay = document.getElement(this.options.overlaySelector);
    var close = this.popup.getElement('.close');
    if (this.options.style_class)
      this.popup.addClass(this.options.style_class);
    this.hide();

    if(close){
      close.addEvent('click', this.hide.bind(this));
    } 
    if(this.options.close_by_overlay){
      this.overlay.addEvent('click', this.hide.bind(this));
      /*HATRED*/
      this.popup.addEvent('click', function(e){
        if(e.target == this.popup){
          this.hide();
        }
      }.bind(this));
    }
    if(this.options.dragable) {
      this.draging = false;
      this.popup.addEvents({
        'mousedown': this.startDrag.bind(this),
        'mouseup': this.stopDrag.bind(this)
      })
      document.addEvent('mousemove', this.move_on_drag.bind(this))
    }
  },

  startDrag: function(e){
    var source = e.target.nodeName.toLowerCase();
    if (source != 'input' && source != 'textarea' && source != 'a') {
      this.coord = e.client;
      this.draging = true;
    }
  },

  stopDrag: function(e){
    this.draging = false;
  },

  move_on_drag: function(e){
    if (this.draging) {
      var coord = e.client;
      var left = parseInt(this.popup.getStyle('left'));
      var top = parseInt(this.popup.getStyle('top'));
      if(isNaN(left) || isNaN(top)) throw("Popup can't be dragged. You must call set_in_centre or set_position method before first show.");
      left += coord.x - this.coord.x;
      top += coord.y - this.coord.y;
      this.coord = coord;

      this.popup.setStyles({'top' : top, 'left': left})
    }
  },

  show: function(){
    this.popup.removeClass('hide');
    this.overlay.removeClass('hide'); 

    /* HATRED*/
    /* hack for IE - force redraw, to show popup second time */
    if(Browser.Engine.trident) {
      var scroll = document.getScroll();
      var y = (scroll.y > 0) ? scroll.y - 1 : scroll.y + 1;
      window.scrollTo(scroll.x, y);
      window.scrollTo(scroll.x, scroll.y);
    }
    if (this.options.adopt_size) {
      this.adopt_size();
    }
    return this;
  },

  hide: function(){
    this.popup.addClass('hide');
    this.overlay.addClass('hide');
    this.fireEvent('close');
    return this;
  },

  destroy: function(){
    this.popup.destroy();
  },
  
  _set_content: function(block, content) {
    if (typeof(content) == 'string'){
      var block_content = (block.html) ? block.html : '';
      block.set('html', block_content+content);
    } else {
      content.inject(block);
    }
  },

  add_title: function(content){
    this._set_content(this.popup_title, content);
    return this;
  },

  add_body: function(content){
    this._set_content(this.popup_body, content);
    return this;
  },

  add_img: function(content){
    this._set_content(this.popup_img, content);
    return this;
  },

  add_img_description: function(content){
    this._set_content(this.popup_img_description, content);
    return this;
  },

  set_title: function(content){
    this.clear_title();
    this.add_title(content);
    return this;
  },

  set_body: function(content){
    this.clear_body();
    this.add_body(content);
    return this;
  },

  /* XXX HACK */
  message: function(message, cls){
    cls = cls? ' ' + cls: '';
    var content = '<div class="simplepopup' + cls + '">' + message + '</div>';
    this.set_body(content);
    return this;
  },

  set_img: function(content){
    this.clear_img();
    this.add_img(content);
    return this;
  },

  set_img_description: function(content){
    this.clear_img_description();
    this.add_img_description(content);
    return this;
  },

  _clear_block: function(block){
    block.set('html', '');
    block.set('text', '');
  },

  clear_title: function(){
    this._clear_block(this.popup_title)
  },

  clear_body: function(){
    this._clear_block(this.popup_body)
  },

  clear_img: function(){
    this._clear_block(this.popup_img);
  },

  clear_img_description: function(){
    this._clear_block(this.popup_img_description);
  },


  clear: function(){
    this.clear_title();
    this.clear_body();
    this.clear_img();
    this.clear_img_description();
    return this;
  },

  _set_position: function(pos){
    this.popup.setStyles(pos);
  },

  set_position: function(pos){
    if(pos.top != undefined) pos.top += 20;
    if(pos.bottom != undefined) pos.bottom += 20;
    if(pos.left != undefined) pos.left += 20;
    if(pos.right != undefined) pos.right += 20;
    this._set_position(pos);
    return this;
  },

  set_in_centre: function(){
    var window_size = window.getSize();
    var popup_size = this.popup.getSize();
    var width_delta = window_size.x - popup_size.x;
    var height_delta = window_size.y - popup_size.y;
    this.popup.setStyle('left', Math.ceil(width_delta/2));
    var scroll_y = 0;
    if (window.pageYOffset) {
        scroll_y = window.pageYOffset;
    } else if (document.documentElement && document.documentElement.scrollTop){  // Explorer 6 Strict
        scroll_y = document.documentElement.scrollTop;
    } else if (document.body) {// all other Explorers
        yScroll = document.body.scrollTop;
    }
    this.popup.setStyle('top', scroll_y + Math.ceil(height_delta/2));
    return this;
  },

  set_width: function(width){
    this.popup.setStyle('width', width);
    return this;
  },

  adopt_size: function() {
    this.set_width('');
    var body_size = this.popup_body.getSize();
    var images = this.popup.getElements('img').map(
        function(_) {return _.src});
    var _post_image_loaded = function () {
      if (body_size.x > this.options.min_width) {
        this.set_width(body_size.x + 40);
      }
      this.set_in_centre();
    }.bind(this);
    if (images.length > 0) {
      Asset.images(images, {
        onComplete: function() {
          _post_image_loaded()
        }.bind(this)
      });
      return;
    }
    _post_image_loaded()
  }

});


var PhotosetPopup = new Class({

  Extends: Popup,

  initialize: function(options) {
    this.parent(options);
    this.clear();
    this.popup.addClass('foto');
    this.archive = options.archive;
    this.switcher = this.popup.getElement('.switch');
    this.download = this.popup.getElement('.download');
    this.download.empty();
    if (this.archive.url) {
      var el = this.add_file(this.archive.url, 'скачать всю галерею', this.archive.ext);
      this._all_galery_el = el.clone();
    }
    this.photos = options.photos;
    this._cached_images = {};
    this.current_index = 0;
  },
  
  add_file: function(file, title/*, ext */) {
    if (typeof(file)=='string'){
      var ext = arguments.length>2? arguments[2]: file.split('.')[1];
      var el = new Element('ul').adopt(
        new Element('li', {'class': ext}).adopt(
          new Element('a', {'href': file,
                            'text': title || 'скачать'})
        )
      )
    } else {
      var el = file;
    }
    el.inject(this.download);
    this.download.removeClass('hide');
    return el;
  },

  clear_download: function(){
    this._clear_block(this.download);
  },

  show: function(){
    this.set_title(this.options.title);
    this.img_tag = new Element('img', {'src': ''});
    this.set_img(this.img_tag);
    this.set_photo(this.current_index, true);
    this.current_asset = null;
    
    return this.parent();
  },

  show_photo: function(id){
    var index = undefined;
    this.photos.some(function(el, i){if (el.id==id) {index=i; return true}});

    this.current_index = index;
    this.set_photo(this.current_index, true);
    // TODO: Add check for index in range
    this.show();
  },

  _show_image: function(image, title, index, need_adoptation) {
    this.set_img(image);
    this.set_body(new Element('p', {'text': title}));
    this.set_switcher(index);
    if (need_adoptation) {
      this.adopt_size();
    }
  },

  set_photo: function(index, need_adoptation) {
    this.switcher.empty();
    this.switcher.adopt(new Element('img', {'src': '/static/img/loader.gif'}));
    var photo = this.photos[index];
    if (photo.src in this._cached_images) {
      this._show_image(this._cached_images[photo.src], photo.title,
                       index, need_adoptation);
    }
    var image = Asset.image(photo.src, {
      onLoad: function(image) {
        this._cached_images[photo.src] = image;
        this._show_image(image, photo.title, index, need_adoptation);
      }.bind(this)
    });

    // Download block
    this.clear_download();
    if (this._all_galery_el)
      this.add_file(this._all_galery_el);
    if (this.photos[index].hd)
      this.add_file(this.photos[index].hd, 'скачать в высоком разрешении');


  },

  set_switcher: function(index) {
    this.switcher.empty();
    new Element('span#go_back', {
      'events': {
        'click': function(e) {
          e.stop();
          if (index>0)
            this.set_photo(index-1);
          else
            this.set_photo(this.photos.length-1);
        }.bind(this)
      }
    }).inject(this.switcher);
    new Element('span', {
      'class': 'num',
      'text': (index+1) + '/' + this.photos.length
    }).inject(this.switcher);
    new Element('span#go_forward', {
      'events': {
        'click': function(e) {
          e.stop();
          if (index < this.photos.length - 1)
            this.set_photo(index+1)
          else
            this.set_photo(0)
        }.bind(this)
      }
    }).inject(this.switcher);
  }

});


var FlashPopup = new Class({

  Extends: Popup,

  initialize: function(options) {
    this.parent(options);
    this.clear();
    this.download = this.popup.getElement('.download');
    this.download.empty();
    if (options.file) {
      this.popup.addClass('foto');
      add_file(this.download, options.file, options.file_title);
    } else {
      this.popup.removeClass('foto');
    }
    this.set_title(options.title);
    window[options.player_maker](this, options);
  }

});


var ArticlePopup = new Class({

  Extends: Popup,

  initialize: function(options) {
    this.parent(options);
    this.popup_annonce = this.popup.getElement('.popup_annonce');
    this.popup_date = this.popup.getElement('.date');
    this.scroll_holder = this.popup.getElement('.article');
    this.scrollable_content = this.popup.getElement('.article .scrollable-content');
    this.anchor = this.popup.getElement('.anchor');

    this.scroll = new Scroll($('popup-article').getElement('.article'), 
                             {consider_images: true}
    );
  },
  
  adopt_size: function() {
    this.parent();
    var scrollable_size = this.scrollable_content.getSize();
    var height_delta = this.popup_title.getSize().y+26; //Delta in heights of scroll-container and popup height
    if (scrollable_size.y+height_delta > this.options.min_height){
      var max_height = window.getSize().y - 100 /*popup-margins*/;
      var new_height = scrollable_size.y+height_delta>max_height?max_height:scrollable_size.y+height_delta;
    } else {
      var new_height =  this.options.min_height;
    }
    this.popup.setStyle("height", new_height);
    this.scroll_holder.setStyle("height", new_height-height_delta);

    this.set_in_centre();
    this.scroll._on_resize();
  },

   // Addition methods. Used mostly in articles
  add_annonce: function(content){
    if (this.popup_annonce){
      this._set_content(this.popup_annonce, content);
    }
    return this;
  },

  add_date: function(content){
    if (this.popup_date){
      this._set_content(this.popup_date, content);
    }
    return this;
  },

  clear_annonce: function(){
    if (this.popup_annonce){
      this._clear_block(this.popup_annonce);
    }
  },

  clear_date: function(){
    if (this.popup_date){
      this._clear_block(this.popup_date);
    }
  },
  
  set_annonce: function(content){
    this.clear_annonce();
    this.add_annonce(content);
    return this;
    
  },

  set_date: function(content){
    this.clear_date();
    this.add_date(content);
    return this;
  },

  set_anchor: function(name){
    this.anchor.set('name', name);
  }



})

function make_audio_player(options) {
  var container = new Element('div.audio-container');
  new Swiff('static/swf/audio_player.swf', {
    id: 'audioplayer',
    width: 241,
    height: 21,
    container: container,
    vars: {
      mp3: options.mp3
    }
  });
  return container;
};


function audio_player(popup, options) {
  popup.set_body(make_audio_player(options));
};


function video_player(popup, options) {
  var container = new Element('div#video-container');
  popup.set_body(container);
  if (options.hdfile && options.hdfile.url) {
    new Element('ul').adopt(
      new Element('li', {'class': options.hdfile.ext}).adopt(
        new Element('a', {'href': options.hdfile.url,
                          'text': 'скачать в высоком качестве'})
      )
    ).inject(popup.download);
    popup.download.removeClass('hide');
  }
  new Swiff('static/swf/video_player.swf', {
    id: 'videoplayer',
    width: 480,
    height: 364,
    container: container,
    params: {wMode: 'opaque'},
    vars: {
      file: options.video_file,
      image: options.preview
    }
  });
};


function make_multiscript_player(options) {
  var container = new Element('div#container');
  new Swiff('static/swf/multiscript.swf', {
    id: 'multiscript',
    width: 940,
    height: 550,
    container: container,
    params: {wMode: 'opaque'},
    vars: {
      src: options.src
    }
  });
  return container;
};


function multiscript_player(popup, options) {
  var container = make_multiscript_player(options);
  popup.set_body(container);
};


function add_file(download, file, title) {
  new Element('ul').adopt(
    new Element('li', {'class': file.split('.')[1]}).adopt(
      new Element('a', {'href': file,
                        'text': title || 'скачать'})
    )
  ).inject(download);
  download.removeClass('hide');
};
