/*
document.addEvent('domready', function(){
    window.console = new Element('div').inject(document.body);
    window.console.setStyles({'position': 'absolute', 
                              'top': 100, 
                              'width': '100%', 
                              'height': 200, 
                              'color': 'black',
                              'padding': 10,
                              'background': '#ffc',
                              'overflow': 'scroll'
                             });
//    window.console.log = function(){
      var res = '';
      for (var i=0, l=arguments.length; i<l; i++){
        res += String(arguments[i]) + ' ';
      }
      var cl = window.console.childNodes.length;
      while (cl > 10){
        window.console.firstChild.destroy();
        cl--;
      }
      window.console.adopt(new Element('p', {'text': res}));
    }
});
*/

var Scroll = new Class({
  Implements: [Events, Options],

  options: {
    base_shift: 100,
    knob_min_height: 50,
    consider_images: false,
    // in case of horizontal scrolling
    knob_min_width: 20,
    scroll_content: '.scrollable-content'
  },
  
  content_mistake: 0,
  knob_mistake: 0,

  _focused: false,
  _is_dragging: false,
  _prev_position: 0,
  _prev_touch_x: 0,
  _prev_touch_y: 0,
  _prev_touch_ts: 0,
  _kinetic_v: 0,

  _container_size: {},
  _content_size: {},
  _indicator_size: {},
  _knob_size: {},

  initialize: function(el, options){
    //hack to prevent multiple initialize with one element
    var scroll = el.retrieve('scroll');
    if(scroll){
      scroll._on_resize();
      return scroll
    }
    this.container = $(el);
    this.container.store('scroll', this);
    this.setOptions(options);
    this.content = this.container.getElement(this.options.scroll_content);
    
    this.injection = this.content.clone(false);
    this.injection.addClass('injection');
    this.container.adopt(this.injection);

    this.content.setStyles({'position': 'absolute', 
                            'top': 0});
    this.container.addClass('ontop');

    this.container.addEvents({'mouseenter': this._mouseover_handler.bind(this),
                              'mouseleave': this._mouseout_handler.bind(this),
                              'mousewheel': this._scroll_handler.bind(this),
                              'keydown': this._keypress_handler.bind(this),
                              'focus': this._focus_handler.bind(this)
                             });
    window.addEvent('resize', this._on_resize.bind(this));
    if (this.options.consider_images) {
      var images = this.content.getElements('img').map(
          function(_) {return _.src});
      if (images.length > 0) {
        Asset.images(images, {
          onComplete: function() {
            this._init();
          }.bind(this)
        });
        return;
      }
    }
    this._init();
  },

  _init: function() {
    this.make_indicator(this.get_proportion());
    this.__set_measures();
    this.resize_knob();
    this.add_touch();
  },

  make_indicator: function(proportion){
    this.knob = new Element('div', {'class': 'knob'}).setStyle('height', this.options.knob_min_height);
    this.indicator = new Element('div', {'class': 'scroll'}).adopt(this.knob);
    this.hide_indicator();

    this.knob.addEvent('mousedown', this._start_knob_dragging.bind(this));
    
    document.addEvents({'mousemove': this._knob_dragging.bind(this),
                      'mouseup': this._stop_knob_dragging.bind(this)
                     });
    
    this.container.adopt(this.indicator);
  },

  add_touch: function(){
    this.container.addEvents({
      'touchstart': this._touchstart_handler.bind(this),
      'touchmove': this._touchmove_handler.bind(this),
      'touchend': this._touchend_handler.bind(this)
    })
    this._touch_moved = false;
  },

  get_proportion: function(){
    return (this._content_size.y < this._container_size.y)? 0 : this._container_size.y/this._content_size.y;    
  },

  resize_knob: function(){
    var height = Math.ceil(this.get_proportion() * this._indicator_size.y);
    this.knob.setStyle('height', (height == 0) ? height : (height > this.options.knob_min_height) ? height : this.options.knob_min_height);
    this._knob_size = this.knob.getSize();
  },

  show_indicator: function(){
    this.resize_knob();
    //this.indicator.setStyle('visibility', 'visible');
  },

  hide_indicator: function(e){
    //$(this.indicator).setStyles({'visibility': 'hidden'});
  },

  calculate_knob_shift: function(content_shift){
    var content_size = this._content_size;
    var container_size = this._container_size;
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    //direct logic is: -1 * (indicator_size.y - knob_size.y) / ((content_size.y - container_size.y) / content_shift);
    return (content_size.y == 0 || content_size.y == container_size.y) ? 0 : ((knob_size.y - indicator_size.y) * content_shift / (content_size.y - container_size.y))
  },

  calculate_content_shift: function(knob_shift){
    var content_size = this._content_size;
    var container_size = this._container_size;
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    //direct logic is: -1 * (content_size.y - container_size.y) / (indicator_size.y - knob_size.y) / knob_shift
    return (indicator_size.y == 0 || indicator_size.y == knob_size.y) ? 0 : (knob_shift * (container_size.y - content_size.y) / (indicator_size.y - knob_size.y))
  },

  _move_correction: function(top, mistake_name){
    var real_top = Math.floor(top);
    this[mistake_name] += top - real_top;
    var correction = Math.floor(this[mistake_name]);
    this[mistake_name] -= correction;
    return real_top + correction;
  },
  
  _move_content_to: function(top){
    var container_size = this._container_size;
    var content_size = this._content_size;
    var at_edge = false;
    top = this._move_correction(top, 'content_mistake');
    top = (top > 0) ? 0 : top;
    top = (top < (container_size.y - content_size.y)) ? (container_size.y - content_size.y) : top;
    top = (container_size.y > content_size.y) ? 0 : top;
    if(top == 0){
      this.container.addClass('ontop');
      this.fireEvent('top');
      at_edge = true;
    } else if (this.container.hasClass('ontop')) {
      // Without this hasClass check it works VERY SLOW on iPad!
      this.container.removeClass('ontop');
    }
    if (top == container_size.y - content_size.y){
      //this.container.addClass('onbottom');
      this.fireEvent('bottom');
      at_edge = true;
    }/* else if (this.container.hasClass('onbottom')) {
      this.container.removeClass('onbottom');
    }*/
    this.content.setStyle('top', top);
    return at_edge;
  },
  
  _move_knob_to: function (top){
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    top = this._move_correction(top, 'knob_mistake');
    top = (top < 0) ? 0 : top;
    top = (top > (indicator_size.y - knob_size.y)) ? (indicator_size.y - knob_size.y) : top;
    this.knob.setStyle('top', top);
  },

  __get_top: function(el){
    return parseInt($(el).getStyle('top'), 10) || 0;
  },

  _shift_content_by: function(shift){
    return this._move_content_to(this.__get_top(this.content) + shift);
  },

  _shift_knob_by: function(shift){
    this._move_knob_to(this.__get_top(this.knob) + shift);
  },

  move_to: function(top, by_knob){
    by_knob = by_knob || false;
    if (by_knob){
      this._move_content_to(this.calculate_content_shift(top));
      this._move_knob_to(top);
    } else {
      this._move_content_to(top);
      this._move_knob_to(this.calculate_knob_shift(top));
    }
  },

  shift_by: function(shift, by_knob){
    by_knob = by_knob || false;
    if (by_knob){
      this._shift_knob_by(shift);
      return this._shift_content_by(this.calculate_content_shift(shift));
    } else {
      this._shift_knob_by(this.calculate_knob_shift(shift));
      return this._shift_content_by(shift);
    }
  },

  __set_measures: function(){
    this._container_size = this.container.getSize();
    this._content_size = this.content.getSize();
    this._indicator_size = this.indicator.getSize();
    this._knob_size = this.knob.getSize();
  },

  __check_active_element: function(){
    // fallback for browsers not supporting activeElement
    return  (!document.activeElement || !['TEXTAREA', 'INPUT', 'SELECT', 'OPTION'].contains(document.activeElement.tagName))
  },

  _mouseover_handler: function(e){
    //focus and tabindex is a hack to get keyboard events on non-input element
    if(!this._focused && this.__check_active_element()){
      this.container.focus();
      this.container.fireEvent('focus');
    }
    this.show_indicator();
  },
  
  _focus_handler: function(){
    this.container.setProperty('tabIndex', 0);
    this.show_indicator();
    this._focused = true;
  },

  _mouseout_handler: function(e){
    this._focused = false;
    this.container.blur();
    if (!this._is_dragging) {
      this.hide_indicator();
    }
  },

  _scroll_handler: function(e){
    if(this.__check_active_element()){
      e.stop();
      // fix for Mac touchpad scroll
      // if (0 && e.event.wheelDelta){
      //  var sh = Math.abs(e.event.wheelDelta) * e.wheel / Math.abs(e.wheel);
      //} else {
      var sh = e.wheel * this.options.base_shift;
      //}
      this.shift_by(sh);
    }
  },

  _keypress_handler: function(e){
    //home
    if(this.__check_active_element()){
      if(e.code == 36){
        this.move_to(0);
      }

      //end
      if(e.code == 35){
        var position = (this._content_size.y < this._container_size.y) ? 0 : (this._container_size.y - this._content_size.y);
        this.move_to(position);
      }

      //up arrow
      if(e.code == 38){
        this.shift_by(this.options.base_shift);
      }
      
      //down arrow
      if(e.code == 40){
        this.shift_by(-1 * this.options.base_shift);
      }
      
      //page up
      if(e.code == 33){
        this.shift_by(this._container_size.y * 0.9);
      }

      //page down
      if(e.code == 34){
        this.shift_by(-1 * this._container_size.y * 0.9);
      }
    }
  },

  _on_resize: function(){
    this.__set_measures();
    this.resize_knob();
    this.move_to(this.__get_top(this.content));
  },


  _start_knob_dragging: function(e){
    e.stop();
    this._is_dragging = true;
    this._prev_position = e.client.y;
  },

  _stop_knob_dragging: function(e){
    //e.stop(); // In chrome and Safarin under Linux and MacOs this prevent flash fron catching mousUp event
    e.stopPropagation();
    this._is_dragging = false;
    if(!this._focused){
      this.hide_indicator();
    }
  },

  _knob_dragging: function(e){
    if(this._is_dragging === true){
      e.stop();
      this.shift_by(e.client.y - this._prev_position, true);
      this._prev_position = e.client.y;
    }
  }, 

  scroll_animation: null,

  scroll_to_element: function(el, speed){
    speed = speed || 500;
    var max_shifts = 25;

    if(typeof el == 'string'){
      el = this.content.getElement(el);
    }
    if(!el){
      throw 'el is ' + el + ', must be an element or selector';
    }

    var total_shift = this.container.getPosition()['y'] - el.getPosition()['y'] + 30;
    var direction = (total_shift > 0) ? 1 : -1;

    if(this.scroll_animation){
      clearInterval(this.scroll_animation);
    }

    while (total_shift/max_shifts*direction < this.options.base_shift && max_shifts > 5){
        max_shifts -= 10;
    } // XXX
    var shift_by = total_shift/max_shifts;
    
    var shift_count = 1;
    this.scroll_animation = setInterval(function(){
      if(shift_count++ > max_shifts){
        clearInterval(this.scroll_animation);
        this.fireEvent('scroll_stop');
        return
      }
      this.shift_by(shift_by);

    }.bind(this), Math.floor(Math.min(speed/max_shifts, 100)));
  },

  is_visible: function(el){
    var cc = this.container.getCoordinates();
    var ce = el.getCoordinates();
    return (ce.top > cc.top && ce.top + ce.height <  cc.top + cc.height) ||
           (ce.top < cc.top && ce.top + ce.height >  cc.top + 200);
  },

  _save_touch: function(e){
    var y = e.client.y, ts = Date.now();
    if (this._prev_touch_y && this._prev_touch_ts && ts-this._prev_touch_ts>5){
      this._kinetic_v = (this._kinetic_v + (y-this._prev_touch_y)/(ts-this._prev_touch_ts)) /2;
    }
    this._prev_touch_x = e.client.x;
    this._prev_touch_y = y;
    this._prev_touch_ts = ts;
  },

  _start_kinetic: function(){
    this.show_indicator();
    if (this._kinetic_timer!=null){
      clearInterval(this._kinetic_timer);
    }
    this._kinetic_timer = this._kinetic_frame.periodical(50, this);
  },

  _stop_kinetic: function(){
    this.hide_indicator();
    if (this._kinetic_timer){
      clearInterval(this._kinetic_timer);
      this._kinetic_timer = null;
    }
    this._kinetic_v = 0;
  },

  _kinetic_frame: function(){
    this._kinetic_v *= 0.95;
    if (Math.abs(this._kinetic_v)<0.02 || this.shift_by(this._kinetic_v*50)){
      this._stop_kinetic();
    }
  },

  _touchstart_handler: function(e){
    this._stop_kinetic()
    this._save_touch(e);
    this.show_indicator();
    this._touch_moved = false;
  },

  _touchmove_handler: function(e){
    if (e.touches==undefined || e.touches.length==1) {
      e.stop();
      this.shift_by((e.client.y - this._prev_touch_y));
      this._save_touch(e);
      this._touch_moved = true;
    }
  },

  _touchend_handler: function(e){
    this.hide_indicator();
    if (this._touch_moved) {
      this._start_kinetic();
    } else {
      var b = document.createEvent("MouseEvents");
      b.initEvent("click", true, false);
      b.isManufactured = true;
      e.target.dispatchEvent(b);
    }
    this._touch_moved = false;
  },

  recalculate_knob_position: function(){
    this.__set_measures();
    this.resize_knob();
    this._move_knob_to(this.calculate_knob_shift(this.__get_top(this.content)));
  },
  
  add_content: function(content, position, element){
    position = position || 'bottom';
    element = element || this.content;

    if (position == 'bottom') {
      content.inject(element);
      this.recalculate_knob_position();
    }

    // copy tree structure for insert into any element inside content block
    function clone_containers(el, top){
      function get_containers(el, top){
        var parent = el.getParent();
        
        if (parent != top) {
          var grandparents = get_containers(parent, top);
          var clone = parent.clone(false);
          clone.setStyles({'padding-top': 0, 'margin-top': 0,
                           'padding-bottom': 0, 'margin-bottom': 0}); 
          grandparents.push(clone);
          return grandparents
        } else {
          return []
        }
      }

      function build_containers(containers, build, lowest){
        var cont = containers.pop();
        if(cont){
          lowest = lowest || cont;
          build = (build) ? cont.grab(build) : cont
          return build_containers(containers, build, lowest);
        }
        return {'containers': build, 'inject_to': lowest}
      }
      return build_containers(get_containers(el, top));
    }
    
    if(position == 'before'){
      if(!element || element == this.content){
        throw "Can not insert outside of scroll content block";
        return
      }
      containers = clone_containers(element, this.content);

      this.injection.empty();
      containers['inject_to'].adopt(content);
      this.injection.adopt(containers['containers']);
      this.content.setStyle('top', this.__get_top(this.content) - this.injection.getSize()['y']);
      content.inject(element, 'before'); 
      this.recalculate_knob_position();
    }



    if (position == 'top') {
      if(element.firstChild){
        this.add_content(content, 'before', element.firstChild);
      } else {
        this.add_content(content, 'bottom', element);
      }
    }

  }
});


var HorizontalScroll = new Class({

  Extends: Scroll,

  __get_top: function(el){
    return parseInt($(el).getStyle('left'), 10) || 0;
  },

  calculate_content_shift: function(knob_shift){
    var content_size = this._content_size;
    var container_size = this._container_size;
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    //direct logic is: -1 * (content_size.y - container_size.y) / (indicator_size.y - knob_size.y) / knob_shift
    return (indicator_size.x == 0 || indicator_size.x == knob_size.x) ? 0 : (knob_shift * (container_size.x - content_size.x) / (indicator_size.x - knob_size.x))
  },

  calculate_knob_shift: function(content_shift){
    var content_size = this._content_size;
    var container_size = this._container_size;
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    //direct logic is: -1 * (indicator_size.y - knob_size.y) / ((content_size.y - container_size.y) / content_shift);
    return (content_size.x == 0 || content_size.x == container_size.x) ? 0 : ((knob_size.x - indicator_size.x) * content_shift / (content_size.x - container_size.x))
  },

  _move_knob_to: function (left){
    var indicator_size = this._indicator_size;
    var knob_size = this._knob_size;
    left = this._move_correction(left, 'knob_mistake');
    left = (left < 0) ? 0 : left;
    left = (left > (indicator_size.x - knob_size.x)) ? (indicator_size.x - knob_size.x) : left;
    this.knob.setStyle('left', left);
  },

  _move_content_to: function(left){
    var container_size = this._container_size;
    var content_size = this._content_size;
    var at_edge = false;
    left = this._move_correction(left, 'content_mistake');
    left = (left > 0) ? 0 : left;
    left = (left < (container_size.x - content_size.x)) ? (container_size.x - content_size.x) : left;
    left = (container_size.x > content_size.x) ? 0 : left;
    if(left == 0){
      this.container.addClass('ontop');
      this.fireEvent('left');
      at_edge = true;
    } else if (this.container.hasClass('ontop')) {
      // Without this hasClass check it works VERY SLOW on iPad!
      this.container.removeClass('ontop');
    }
    if (left == container_size.x - content_size.x){
      this.fireEvent('bottom');
      at_edge = true;
    }
    this.content.setStyle('left', left);
    return at_edge;
  },

  _start_knob_dragging: function(e){
    e.stop();
    this._is_dragging = true;
    this._prev_position = e.client.x;
  },

  _knob_dragging: function(e){
    if(this._is_dragging === true){
      e.stop();
      this.shift_by(e.client.x - this._prev_position, true);
      this._prev_position = e.client.x;
    }
  }, 

  get_proportion: function(){
    return (this._content_size.x < this._container_size.x)? 0 : this._container_size.x/this._content_size.x;
  },

  resize_knob: function(){
    var width = Math.ceil(this.get_proportion() * this._indicator_size.x);
    this.knob.setStyle('width', (width == 0) ? width : (width > this.options.knob_min_width) ? width : this.options.knob_min_width);
    this._knob_size = this.knob.getSize();
  },

  make_indicator: function(proportion){
    this.knob = new Element('div', {'class': 'knob'});
    this.indicator = new Element('div', {'class': 'scroll'}).adopt(this.knob);
    this.hide_indicator();

    this.knob.addEvent('mousedown', this._start_knob_dragging.bind(this));
    document.addEvents({'mousemove': this._knob_dragging.bind(this),
                        'mouseup': this._stop_knob_dragging.bind(this)});
    this.container.adopt(this.indicator);
  }

});
